@@ -62,6 +62,11 @@ class MESI(Protocol):
         if event == Ops.PrWr:
             self.perform_processor_operation(pid, function)
 
+        # Self Invalidate
+        if event == Ops.PrInv:
+            self.set_processor_state(pid, "I")
+            self.set_processor_cache_content(pid, None)
+
         # Invalidate copy
         if event in (Ops.BusUpgr, Ops.BusRdX):
             self.set_processor_cache_content(pid, None)
@@ -98,4 +103,4 @@ def example1():
 # Main
 ###################
 # example1()
-# mesi.print(_filter="transaction")
+# example2()
