@@ -47,6 +47,11 @@ class MSI(Protocol):
         if event == Ops.PrWr:
             self.perform_processor_operation(pid, function)
 
+        # Self Invalidate
+        if event == Ops.PrInv:
+            self.set_processor_state(pid, "I")
+            self.set_processor_cache_content(pid, None)
+
         # Invalidate copy
         if event in (Ops.BusUpgr, Ops.BusRdX):
             self.set_processor_cache_content(pid, None)
@@ -74,10 +79,12 @@ def example1():
         (2, Ops.PrRd),
         (2, Ops.PrWr, lambda x: x + 2),
         (1, Ops.PrWr, lambda x: x * 2),
+        # (1, Ops.PrInv),
         (2, Ops.PrRd),
     ]
     msi = MSI(n_processors=2, memory_content=3)
     msi.perform_instructions(instructions)
+    print(msi.statistics())
 
 ###################
 # Main
