@@ -1,6 +1,7 @@
 from MSI import MSI
 from MESI import MESI
 from Operations import MSIOps as Ops
+from CacheCoherence import protocols
 
 
 ###################
@@ -116,6 +117,23 @@ def example6():
     msi.perform_instructions(instructions)
 
 
+###################
+# http://people.ee.duke.edu/~sorin/prior-courses/ece152-spring2009/lectures/8.3-multicore.pdf
+###################
+def example7():
+    instructions = [
+        (1, Ops.PrRd),
+        (3, Ops.PrRd),
+        (3, Ops.PrWr, lambda x: x + 2),
+        (1, Ops.PrRd),
+        (2, Ops.PrRd),
+    ]
+    for name, protocol in protocols.items():
+        p = protocol(n_processors=3, memory_content=3)
+        p.perform_instructions(instructions, display=False)
+        print(p.statistics())
+
+
 ###################
 # Main
 ###################
@@ -125,3 +143,4 @@ def example6():
 # example4()
 # example5()
 # example6()
+# example7()
