@@ -1,6 +1,7 @@
 from tabulate import tabulate
 from .Processor import Processor
-from .Operations import MSIOps as Ops
+from .Operations import MESIOps as Ops
+from .States import MESIState as State
 
 
 class Protocol:
@@ -12,10 +13,11 @@ class Protocol:
         self.n_reads = 0
         self.n_writes = 0
         self.n_state_changes = 0
+        self.n_invalidations = 0
         self.n_bus_latency = 0
         self.n_cache_misses = 0
-        self.n_invalidations = 0
         self.n_cache_to_cache_transfers = 0
+
         # Additional Private Variables
         self.modified_value = None
         self.history = []
@@ -31,6 +33,15 @@ class Protocol:
             self.n_reads += 1
         elif event == Ops.PrWr:
             self.n_writes += 1
+        state = self.get_processor_state(pid)
+        if state == State.I:
+            states = {p.state for p in self.other_processors_not(pid=pid)}
+            if len(states) == 1 and State.I in states:
+                self.n_cache_misses += 1
+            if State.S in states:
+                self.n_cache_misses += 1
+            if states.intersection({State.M, State.E}):
+                self.n_cache_to_cache_transfers += 1
 
     def get_headers(self):
         headers = ["Step"]
@@ -47,6 +58,10 @@ class Protocol:
     def processor(self, pid):
         return self.processors[pid - 1]
 
+    def other_processors_not(self, pid):
+        """Gets the list of other processors which are not equal to pid"""
+        return [p for p in self.processors if p.pid != pid]
+
     def get_processor_state(self, pid):
         return self.processor(pid).state
 
@@ -93,11 +108,11 @@ class Protocol:
         step_name = self.format_instruction_name(pid, event)
         self.save_history(step_name, main_transaction)
 
-    def perform_instructions(self, instructions):
+    def perform_instructions(self, instructions, display=True):
         for i in instructions:
             pid, operation, function = i[0], i[1], i[2] if len(i) == 3 else None
             self._perform_instruction(pid, operation, function)
-        print(self, '\n')
+        if display: print(self, '\n')
 
     ###########################
     # Statistical Methods
@@ -110,7 +125,8 @@ class Protocol:
             "Bus Transactions": self.n_bus_latency,
             "States": self.n_state_changes,
             "Invalidations": self.n_invalidations,
-            # TODO "Cache-to-Cache transfers": self.n_cache_to_cache_transfers,
+            "Cache-to-Cache Transfers": self.n_cache_to_cache_transfers,
+            "Cache Misses": self.n_cache_misses,
         }
         return stats
 
